From: Herve Codina <herve.codina@bootlin.com>
To: Andrew Lunn <andrew@lunn.ch>, Rob Herring <robh@kernel.org>,
Krzysztof Kozlowski <krzk+dt@kernel.org>,
Conor Dooley <conor+dt@kernel.org>,
Geert Uytterhoeven <geert+renesas@glider.be>,
Kalle Niemi <kaleposti@gmail.com>,
Matti Vaittinen <mazziesaccount@gmail.com>,
Greg Kroah-Hartman <gregkh@linuxfoundation.org>,
"Rafael J. Wysocki" <rafael@kernel.org>,
Danilo Krummrich <dakr@kernel.org>, Frank Li <Frank.Li@nxp.com>,
Sascha Hauer <s.hauer@pengutronix.de>,
Pengutronix Kernel Team <kernel@pengutronix.de>,
Fabio Estevam <festevam@gmail.com>,
Michael Turquette <mturquette@baylibre.com>,
Stephen Boyd <sboyd@kernel.org>,
Andi Shyti <andi.shyti@kernel.org>,
Wolfram Sang <wsa+renesas@sang-engineering.com>,
Peter Rosin <peda@axentia.se>, Arnd Bergmann <arnd@arndb.de>,
Herve Codina <herve.codina@bootlin.com>,
Saravana Kannan <saravanak@kernel.org>,
Bjorn Helgaas <bhelgaas@google.com>,
Charles Keepax <ckeepax@opensource.cirrus.com>,
Richard Fitzgerald <rf@opensource.cirrus.com>,
David Rhodes <david.rhodes@cirrus.com>,
Linus Walleij <linusw@kernel.org>,
Ulf Hansson <ulf.hansson@linaro.org>,
Mark Brown <broonie@kernel.org>, Len Brown <lenb@kernel.org>,
Andy Shevchenko <andriy.shevchenko@linux.intel.com>,
Daniel Scally <djrscally@gmail.com>,
Heikki Krogerus <heikki.krogerus@linux.intel.com>,
Sakari Ailus <sakari.ailus@linux.intel.com>,
Davidlohr Bueso <dave@stgolabs.net>,
Jonathan Cameron <jonathan.cameron@huawei.com>,
Dave Jiang <dave.jiang@intel.com>,
Alison Schofield <alison.schofield@intel.com>,
Vishal Verma <vishal.l.verma@intel.com>,
Ira Weiny <ira.weiny@intel.com>,
Dan Williams <dan.j.williams@intel.com>,
Shawn Guo <shawnguo@kernel.org>
Cc: Wolfram Sang <wsa@kernel.org>,
linux-kernel@vger.kernel.org, driver-core@lists.linux.dev,
imx@lists.linux.dev, linux-arm-kernel@lists.infradead.org,
linux-clk@vger.kernel.org, linux-i2c@vger.kernel.org,
devicetree@vger.kernel.org, linux-pci@vger.kernel.org,
linux-sound@vger.kernel.org, patches@opensource.cirrus.com,
linux-gpio@vger.kernel.org, linux-pm@vger.kernel.org,
linux-spi@vger.kernel.org, linux-acpi@vger.kernel.org,
linux-cxl@vger.kernel.org,
Allan Nielsen <allan.nielsen@microchip.com>,
Horatiu Vultur <horatiu.vultur@microchip.com>,
Steen Hegelund <steen.hegelund@microchip.com>,
Luca Ceresoli <luca.ceresoli@bootlin.com>,
Thomas Petazzoni <thomas.petazzoni@bootlin.com>
Subject: [PATCH v6 05/27] bus: simple-pm-bus: Populate child nodes at probe
Date: Wed, 25 Mar 2026 15:35:32 +0100 [thread overview]
Message-ID: <20260325143555.451852-6-herve.codina@bootlin.com> (raw)
In-Reply-To: <20260325143555.451852-1-herve.codina@bootlin.com>
The simple-pm-bus driver handles several simple busses. When it is used
with busses other than a compatible "simple-pm-bus", it doesn't populate
its child devices during its probe.
This confuses fw_devlink and results in wrong or missing devlinks.
Once a driver is bound to a device and the probe() has been called,
device_links_driver_bound() is called.
This function performs operation based on the following assumption:
If a child firmware node of the bound device is not added as a
device, it will never be added.
Among operations done on fw_devlinks of those "never be added" devices,
device_links_driver_bound() changes their supplier.
With devices attached to a simple-bus compatible device, this change
leads to wrong devlinks where supplier of devices points to the device
parent (i.e. simple-bus compatible device) instead of the device itself
(i.e. simple-bus child).
When the device attached to the simple-bus is removed, because devlinks
are not correct, its consumers are not removed first.
In order to have correct devlinks created, make the simple-pm-bus driver
compliant with the devlink assumption and create its child devices
during its probe.
Signed-off-by: Herve Codina <herve.codina@bootlin.com>
---
drivers/bus/simple-pm-bus.c | 21 ++++++++++++---------
1 file changed, 12 insertions(+), 9 deletions(-)
diff --git a/drivers/bus/simple-pm-bus.c b/drivers/bus/simple-pm-bus.c
index 9ebd588ea24f..30628958c923 100644
--- a/drivers/bus/simple-pm-bus.c
+++ b/drivers/bus/simple-pm-bus.c
@@ -42,14 +42,15 @@ static int simple_pm_bus_probe(struct platform_device *pdev)
match = of_match_device(dev->driver->of_match_table, dev);
/*
* These are transparent bus devices (not simple-pm-bus matches) that
* have their child nodes populated automatically. So, don't need to
* do anything more. We only match with the device if this driver is
* the most specific match because we don't want to incorrectly bind to
* a device that has a more specific driver.
*/
if (match && match->data) {
if (of_property_match_string(np, "compatible", match->compatible) == 0)
- return 0;
+ goto populate;
else
return -ENODEV;
}
@@ -64,13 +65,14 @@ static int simple_pm_bus_probe(struct platform_device *pdev)
dev_set_drvdata(&pdev->dev, bus);
- dev_dbg(&pdev->dev, "%s\n", __func__);
-
pm_runtime_enable(&pdev->dev);
+populate:
if (np)
of_platform_populate(np, NULL, lookup, &pdev->dev);
+ dev_dbg(&pdev->dev, "%s\n", __func__);
+
return 0;
}
@@ -78,7 +80,7 @@ static void simple_pm_bus_remove(struct platform_device *pdev)
{
const void *data = of_device_get_match_data(&pdev->dev);
- if (pdev->driver_override || data)
+ if (pdev->driver_override)
return;
dev_dbg(&pdev->dev, "%s\n", __func__);
@@ -86,7 +88,8 @@ static void simple_pm_bus_remove(struct platform_device *pdev)
if (pdev->dev.of_node)
of_platform_depopulate(&pdev->dev);
- pm_runtime_disable(&pdev->dev);
+ if (!data)
+ pm_runtime_disable(&pdev->dev);
}
static int simple_pm_bus_runtime_suspend(struct device *dev)
@@ -42,14 +42,15 @@ static int simple_pm_bus_probe(struct platform_device *pdev)
/*
* These are transparent bus devices (not simple-pm-bus matches) that
- * have their child nodes populated automatically. So, don't need to
- * do anything more. We only match with the device if this driver is
- * the most specific match because we don't want to incorrectly bind to
- * a device that has a more specific driver.
+ * need to have their child nodes populated. So, don't need to do
+ * anything more except populate child nodes during this probe(). We
+ * only match with the device if this driver is the most specific match
+ * because we don't want to incorrectly bind to a device that has a more
+ * specific driver.
*/
if (match && match->data) {
--
2.53.0
next prev parent reply other threads:[~2026-03-25 14:37 UTC|newest]
Thread overview: 47+ messages / expand[flat|nested] mbox.gz Atom feed top
2026-03-25 14:35 [PATCH v6 00/27] lan966x pci device: Add support for SFPs Herve Codina
2026-03-25 14:35 ` [PATCH v6 01/27] Revert "treewide: Fix probing of devices in DT overlays" Herve Codina
2026-04-07 18:05 ` Rob Herring (Arm)
2026-04-08 8:03 ` Geert Uytterhoeven
2026-03-25 14:35 ` [PATCH v6 02/27] of: dynamic: Fix overlayed devices not probing because of fw_devlink Herve Codina
2026-04-07 18:06 ` Rob Herring (Arm)
2026-04-22 17:36 ` Geert Uytterhoeven
2026-04-22 18:51 ` Doug Anderson
2026-04-22 19:26 ` Andy Shevchenko
2026-04-22 19:35 ` Doug Anderson
2026-04-23 6:53 ` Geert Uytterhoeven
2026-04-23 8:41 ` Herve Codina
2026-03-25 14:35 ` [PATCH v6 03/27] driver core: Avoid warning when removing a device while its supplier is unbinding Herve Codina
2026-03-25 14:35 ` [PATCH v6 04/27] bus: simple-pm-bus: Remove child devices when the bus is unbound Herve Codina
2026-03-25 14:35 ` Herve Codina [this message]
2026-03-25 14:35 ` [PATCH v6 06/27] driver core: fw_devlink: Introduce fw_devlink_set_device() Herve Codina
2026-03-25 14:35 ` [PATCH v6 07/27] drivers: core: Use fw_devlink_set_device() Herve Codina
2026-03-25 14:35 ` [PATCH v6 08/27] pinctrl: cs42l43: " Herve Codina
2026-03-25 14:35 ` [PATCH v6 09/27] cxl/test: Use device_set_node() Herve Codina
2026-03-25 14:35 ` [PATCH v6 10/27] cxl/test: Use fw_devlink_set_device() Herve Codina
2026-03-25 14:35 ` [PATCH v6 11/27] PCI: of: " Herve Codina
2026-03-25 20:10 ` Bjorn Helgaas
2026-03-25 14:35 ` [PATCH v6 12/27] PCI: of: Set fwnode device of newly created PCI device nodes Herve Codina
2026-03-25 20:13 ` Bjorn Helgaas
2026-03-25 14:35 ` [PATCH v6 13/27] PCI: of: Remove fwnode_dev_initialized() call for a PCI root bridge node Herve Codina
2026-03-25 20:20 ` Bjorn Helgaas
2026-05-06 20:30 ` Herve Codina
2026-03-25 14:35 ` [PATCH v6 14/27] i2c: core: Introduce i2c_get_adapter_physdev() Herve Codina
2026-03-25 14:35 ` [PATCH v6 15/27] i2c: mux: Set adapter physical device Herve Codina
2026-03-25 14:35 ` [PATCH v6 16/27] i2c: mux: Create missing devlink between mux and " Herve Codina
2026-03-25 14:35 ` [PATCH v6 17/27] of: property: Allow fw_devlink device-tree on x86 Herve Codina
2026-04-07 18:09 ` Rob Herring (Arm)
2026-03-25 14:35 ` [PATCH v6 18/27] clk: lan966x: Add MCHP_LAN966X_PCI dependency Herve Codina
2026-03-25 14:35 ` [PATCH v6 19/27] i2c: busses: at91: " Herve Codina
2026-03-25 14:35 ` [PATCH v6 20/27] misc: lan966x_pci: Fix dtso nodes ordering Herve Codina
2026-03-26 10:33 ` Linus Walleij
2026-05-07 7:23 ` Herve Codina
2026-05-07 9:27 ` Linus Walleij
2026-03-25 14:35 ` [PATCH v6 21/27] misc: lan966x_pci: Split dtso in dtsi/dtso Herve Codina
2026-03-25 14:35 ` [PATCH v6 22/27] misc: lan966x_pci: Rename lan966x_pci.dtso to lan966x_evb_lan9662_nic.dtso Herve Codina
2026-03-25 14:35 ` [PATCH v6 23/27] PCI: Add Microchip LAN9662 PCI Device ID Herve Codina
2026-03-25 14:35 ` [PATCH v6 24/27] misc: lan966x_pci: Introduce board specific data Herve Codina
2026-03-25 14:35 ` [PATCH v6 25/27] misc: lan966x_pci: Add dtsi/dtso nodes in order to support SFPs Herve Codina
2026-03-25 14:35 ` [PATCH v6 26/27] misc: lan966x_pci: Sort the drivers list in Kconfig help Herve Codina
2026-03-25 14:35 ` [PATCH v6 27/27] misc: lan966x_pci: Add drivers needed to support SFPs " Herve Codina
2026-04-07 18:03 ` [PATCH v6 00/27] lan966x pci device: Add support for SFPs Rob Herring
2026-04-07 18:05 ` Rob Herring
Reply instructions:
You may reply publicly to this message via plain-text email
using any one of the following methods:
* Save the following mbox file, import it into your mail client,
and reply-to-all from there: mbox
Avoid top-posting and favor interleaved quoting:
https://en.wikipedia.org/wiki/Posting_style#Interleaved_style
* Reply using the --to, --cc, and --in-reply-to
switches of git-send-email(1):
git send-email \
--in-reply-to=20260325143555.451852-6-herve.codina@bootlin.com \
--to=herve.codina@bootlin.com \
--cc=Frank.Li@nxp.com \
--cc=alison.schofield@intel.com \
--cc=allan.nielsen@microchip.com \
--cc=andi.shyti@kernel.org \
--cc=andrew@lunn.ch \
--cc=andriy.shevchenko@linux.intel.com \
--cc=arnd@arndb.de \
--cc=bhelgaas@google.com \
--cc=broonie@kernel.org \
--cc=ckeepax@opensource.cirrus.com \
--cc=conor+dt@kernel.org \
--cc=dakr@kernel.org \
--cc=dan.j.williams@intel.com \
--cc=dave.jiang@intel.com \
--cc=dave@stgolabs.net \
--cc=david.rhodes@cirrus.com \
--cc=devicetree@vger.kernel.org \
--cc=djrscally@gmail.com \
--cc=driver-core@lists.linux.dev \
--cc=festevam@gmail.com \
--cc=geert+renesas@glider.be \
--cc=gregkh@linuxfoundation.org \
--cc=heikki.krogerus@linux.intel.com \
--cc=horatiu.vultur@microchip.com \
--cc=imx@lists.linux.dev \
--cc=ira.weiny@intel.com \
--cc=jonathan.cameron@huawei.com \
--cc=kaleposti@gmail.com \
--cc=kernel@pengutronix.de \
--cc=krzk+dt@kernel.org \
--cc=lenb@kernel.org \
--cc=linusw@kernel.org \
--cc=linux-acpi@vger.kernel.org \
--cc=linux-arm-kernel@lists.infradead.org \
--cc=linux-clk@vger.kernel.org \
--cc=linux-cxl@vger.kernel.org \
--cc=linux-gpio@vger.kernel.org \
--cc=linux-i2c@vger.kernel.org \
--cc=linux-kernel@vger.kernel.org \
--cc=linux-pci@vger.kernel.org \
--cc=linux-pm@vger.kernel.org \
--cc=linux-sound@vger.kernel.org \
--cc=linux-spi@vger.kernel.org \
--cc=luca.ceresoli@bootlin.com \
--cc=mazziesaccount@gmail.com \
--cc=mturquette@baylibre.com \
--cc=patches@opensource.cirrus.com \
--cc=peda@axentia.se \
--cc=rafael@kernel.org \
--cc=rf@opensource.cirrus.com \
--cc=robh@kernel.org \
--cc=s.hauer@pengutronix.de \
--cc=sakari.ailus@linux.intel.com \
--cc=saravanak@kernel.org \
--cc=sboyd@kernel.org \
--cc=shawnguo@kernel.org \
--cc=steen.hegelund@microchip.com \
--cc=thomas.petazzoni@bootlin.com \
--cc=ulf.hansson@linaro.org \
--cc=vishal.l.verma@intel.com \
--cc=wsa+renesas@sang-engineering.com \
--cc=wsa@kernel.org \
/path/to/YOUR_REPLY
https://kernel.org/pub/software/scm/git/docs/git-send-email.html
* If your mail client supports setting the In-Reply-To header
via mailto: links, try the mailto: link
Be sure your reply has a Subject: header at the top and a blank line
before the message body.
This is an external index of several public inboxes,
see mirroring instructions on how to clone and mirror
all data and code used by this external index.